@@ -1,48 +1,100 @@
-fn main() -> webview2_nuget::Result<()> {
+extern crate windows_bindgen;
+
+fn main() -> crate::Result<()> {
     if let Ok(package_root) = webview2_nuget::install() {
         webview2_nuget::update_windows(&package_root)?;
+        webview2_nuget::update_rustc_flags()?;
         webview2_nuget::update_browser_version(&package_root)?;
         webview2_nuget::update_callback_interfaces(&package_root)?;
     }
 
-    windows::runtime::build! {
-        Microsoft::Web::WebView2::Win32::*,
-    };
+    webview2_bindgen::update_bindings()?;
 
     Ok(())
 }
 
+#[derive(Debug, Error)]
+pub enum Error {
+    #[error(transparent)]
+    Io(#[from] std::io::Error),
+    #[error(transparent)]
+    Var(#[from] std::env::VarError),
+    #[error(transparent)]
+    Json(#[from] serde_json::Error),
+    #[error(transparent)]
+    Regex(#[from] regex::Error),
+    #[error("Missing Version")]
+    MissingVersion,
+    #[error("Missing Typedef")]
+    MissingTypedef,
+    #[error("Missing Path")]
+    MissingPath(std::path::PathBuf),
+}
+
+pub type Result<T> = std::result::Result<T, Error>;
+
 #[macro_use]
 extern crate thiserror;
 
+mod webview2_path {
+    use std::{convert::From, env, path::PathBuf, process::Command};
+
+    pub fn get_out_dir() -> super::Result<PathBuf> {
+        Ok(PathBuf::from(env::var("OUT_DIR")?))
+    }
+
+    pub fn get_manifest_dir() -> super::Result<PathBuf> {
+        Ok(PathBuf::from(env::var("CARGO_MANIFEST_DIR")?))
+    }
+
+    pub fn get_workspace_dir() -> super::Result<PathBuf> {
+        use serde::Deserialize;
+
+        #[derive(Deserialize)]
+        struct CargoMetadata {
+            workspace_root: String,
+        }
+
+        let output = Command::new(env::var("CARGO")?)
+            .args(&["metadata", "--format-version=1", "--no-deps", "--offline"])
+            .output()?;
+
+        let metadata: CargoMetadata = serde_json::from_slice(&output.stdout)?;
+
+        Ok(PathBuf::from(metadata.workspace_root))
+    }
+}
+
 mod webview2_nuget {
     use std::{
         convert::From,
-        env, fs,
-        io::{self, Read, Write},
+        fs,
+        io::{Read, Write},
         path::{Path, PathBuf},
         process::Command,
     };
 
     use regex::Regex;
 
+    use super::webview2_path::*;
+
     include!("./src/browser_version.rs");
     include!("./src/callback_interfaces.rs");
 
     const WEBVIEW2_NAME: &str = "Microsoft.Web.WebView2";
-    const WEBVIEW2_VERSION: &str = "1.0.1020.30";
+    const WEBVIEW2_VERSION: &str = "1.0.1054.31";
 
     #[cfg(not(windows))]
-    pub fn install() -> Result<PathBuf> {
+    pub fn install() -> super::Result<PathBuf> {
         get_manifest_dir()
     }
 
     #[cfg(windows)]
-    pub fn install() -> Result<PathBuf> {
+    pub fn install() -> super::Result<PathBuf> {
         let out_dir = get_out_dir()?;
         let install_root = match out_dir.to_str() {
             Some(path) => path,
-            None => return Err(Error::MissingPath(out_dir)),
+            None => return Err(super::Error::MissingPath(out_dir)),
         };
 
         let mut package_root = out_dir.clone();
@@ -55,7 +107,7 @@ mod webview2_nuget {
 
             let nuget_tool = match nuget_path.to_str() {
                 Some(path) => path,
-                None => return Err(Error::MissingPath(nuget_path)),
+                None => return Err(super::Error::MissingPath(nuget_path)),
             };
 
             Command::new(nuget_tool)
@@ -70,23 +122,15 @@ mod webview2_nuget {
                 .output()?;
 
             if !check_nuget_dir(install_root)? {
-                return Err(Error::MissingPath(package_root));
+                return Err(super::Error::MissingPath(package_root));
             }
         }
 
         Ok(package_root)
     }
 
-    fn get_out_dir() -> Result<PathBuf> {
-        Ok(PathBuf::from(env::var("OUT_DIR")?))
-    }
-
-    fn get_manifest_dir() -> Result<PathBuf> {
-        Ok(PathBuf::from(env::var("CARGO_MANIFEST_DIR")?))
-    }
-
     #[cfg(windows)]
-    fn check_nuget_dir(install_root: &str) -> Result<bool> {
+    fn check_nuget_dir(install_root: &str) -> super::Result<bool> {
         let nuget_path = format!("{}.{}", WEBVIEW2_NAME, WEBVIEW2_VERSION);
         let mut dir_iter = fs::read_dir(install_root)?.filter(|dir| match dir {
             Ok(dir) => match dir.file_type() {
@@ -105,12 +149,12 @@ mod webview2_nuget {
     }
 
     #[cfg(not(windows))]
-    pub fn update_windows(_: &Path) -> Result<()> {
+    pub fn update_windows(_: &Path) -> super::Result<()> {
         Ok(())
     }
 
     #[cfg(windows)]
-    pub fn update_windows(package_root: &Path) -> Result<()> {
+    pub fn update_windows(package_root: &Path) -> super::Result<()> {
         const WEBVIEW2_STATIC_LIB: &str = "WebView2LoaderStatic.lib";
         const WEBVIEW2_TARGETS: &[&str] = &["arm64", "x64", "x86"];
 
@@ -141,33 +185,48 @@ mod webview2_nuget {
             fs::copy(lib_src.as_path(), lib_dest.as_path())?;
         }
 
+        println!("cargo:rerun-if-changed={}", bindings_windows_dir.display());
+
         Ok(())
     }
 
-    fn get_workspace_dir() -> Result<PathBuf> {
-        use serde::Deserialize;
+    #[cfg(not(windows))]
+    pub fn update_rustc_flags() -> super::Result<()> {
+        Ok(())
+    }
 
-        #[derive(Deserialize)]
-        struct CargoMetadata {
-            workspace_root: String,
-        }
+    #[cfg(windows)]
+    pub fn update_rustc_flags() -> super::Result<()> {
+        let mut lib_path = get_manifest_dir()?;
+        lib_path.push(".windows");
 
-        let output = Command::new(env::var("CARGO")?)
-            .args(&["metadata", "--format-version=1", "--no-deps", "--offline"])
-            .output()?;
+        let target_arch = match ::std::env::var("CARGO_CFG_TARGET_ARCH")?.as_str() {
+            "x86_64" => "x64",
+            "x86" => "x86",
+            "arm" => "arm",
+            "aarch64" => "arm64",
+            unimplemented => unimplemented!(
+                "`{}` architecture set by `CARGO_CFG_TARGET_ARCH`",
+                unimplemented
+            ),
+        };
+        lib_path.push(target_arch);
 
-        let metadata: CargoMetadata = serde_json::from_slice(&output.stdout)?;
+        match lib_path.to_str() {
+            Some(path) if lib_path.exists() => println!("cargo:rustc-link-search=native={}", path),
+            _ => unimplemented!("`{}` is not supported by WebView2", target_arch),
+        };
 
-        Ok(PathBuf::from(metadata.workspace_root))
+        Ok(())
     }
 
     #[cfg(not(windows))]
-    pub fn update_browser_version(_: &PathBuf) -> Result<bool> {
+    pub fn update_browser_version(_: &PathBuf) -> super::Result<bool> {
         Ok(false)
     }
 
     #[cfg(windows)]
-    pub fn update_browser_version(package_root: &Path) -> Result<bool> {
+    pub fn update_browser_version(package_root: &Path) -> super::Result<bool> {
         let version = get_target_broweser_version(package_root)?;
         if version == CORE_WEBVIEW_TARGET_PRODUCT_VERSION {
             return Ok(false);
@@ -186,7 +245,7 @@ mod webview2_nuget {
     }
 
     #[cfg(windows)]
-    fn get_target_broweser_version(package_root: &Path) -> Result<String> {
+    fn get_target_broweser_version(package_root: &Path) -> super::Result<String> {
         let mut include_path = package_root.to_path_buf();
         include_path.push("build");
         include_path.push("native");
@@ -203,17 +262,17 @@ mod webview2_nuget {
             .next()
         {
             Some(capture) => Ok(capture.as_str().into()),
-            None => Err(Error::MissingVersion),
+            None => Err(super::Error::MissingVersion),
         }
     }
 
     #[cfg(not(windows))]
-    pub fn update_callback_interfaces(_: &PathBuf) -> Result<bool> {
+    pub fn update_callback_interfaces(_: &PathBuf) -> super::Result<bool> {
         Ok(false)
     }
 
     #[cfg(windows)]
-    pub fn update_callback_interfaces(package_root: &Path) -> Result<bool> {
+    pub fn update_callback_interfaces(package_root: &Path) -> super::Result<bool> {
         let interfaces = get_callback_interfaces(package_root)?;
         let declared = all_declared().into_iter().map(String::from).collect();
         if interfaces == declared {
@@ -247,7 +306,7 @@ pub fn all_declared() -> BTreeSet<&'static str> {{
     }
 
     #[cfg(windows)]
-    fn get_callback_interfaces(package_root: &Path) -> Result<BTreeSet<String>> {
+    fn get_callback_interfaces(package_root: &Path) -> super::Result<BTreeSet<String>> {
         let mut include_path = package_root.to_path_buf();
         include_path.push("build");
         include_path.push("native");
@@ -264,29 +323,78 @@ pub fn all_declared() -> BTreeSet<&'static str> {{
             .map(|match_1| String::from(match_1.as_str()))
             .collect();
         if interfaces.is_empty() {
-            Err(Error::MissingTypedef)
+            Err(super::Error::MissingTypedef)
         } else {
             Ok(interfaces)
         }
     }
+}
 
-    #[derive(Debug, Error)]
-    pub enum Error {
-        #[error(transparent)]
-        Io(#[from] io::Error),
-        #[error(transparent)]
-        Var(#[from] env::VarError),
-        #[error(transparent)]
-        Json(#[from] serde_json::Error),
-        #[error(transparent)]
-        Regex(#[from] regex::Error),
-        #[error("Missing Version")]
-        MissingVersion,
-        #[error("Missing Typedef")]
-        MissingTypedef,
-        #[error("Missing Path")]
-        MissingPath(PathBuf),
+mod webview2_bindgen {
+    use std::{
+        fs,
+        io::{Read, Write},
+        path::{Path, PathBuf},
+    };
+
+    use windows_bindgen::{gen_namespace, Gen};
+
+    use super::webview2_path::*;
+
+    #[cfg(not(windows))]
+    pub fn update_bindings() -> super::Result<bool> {
+        Ok(false)
     }
 
-    pub type Result<T> = std::result::Result<T, Error>;
+    #[cfg(windows)]
+    pub fn update_bindings() -> super::Result<bool> {
+        let source_path = generate_bindings()?;
+        format_bindings(&source_path)?;
+        let source = read_bindings(&source_path)?;
+
+        let mut dest_path = get_manifest_dir()?;
+        dest_path.push("src");
+        dest_path.push("Microsoft");
+        dest_path.push("Web");
+        dest_path.push("WebView2");
+        dest_path.push("Win32");
+        dest_path.push("mod.rs");
+        let dest = read_bindings(&dest_path)?;
+
+        if source != dest {
+            fs::copy(&source_path, &dest_path)?;
+            Ok(true)
+        } else {
+            Ok(false)
+        }
+    }
+
+    #[cfg(windows)]
+    fn generate_bindings() -> super::Result<PathBuf> {
+        let mut source_path = get_out_dir()?;
+        source_path.push("mod.rs");
+        let mut source_file = fs::File::create(source_path.clone())?;
+        let gen = Gen {
+            namespace: "Microsoft.Web.WebView2.Win32",
+            ..Default::default()
+        };
+        source_file.write_all(gen_namespace(&gen).as_bytes())?;
+        Ok(source_path)
+    }
+
+    #[cfg(windows)]
+    fn format_bindings(source_path: &Path) -> super::Result<()> {
+        let mut cmd = ::std::process::Command::new("rustfmt");
+        cmd.arg(&source_path);
+        cmd.output()?;
+        Ok(())
+    }
+
+    #[cfg(windows)]
+    fn read_bindings(source_path: &Path) -> super::Result<String> {
+        let mut source_file = fs::File::open(source_path)?;
+        let mut updated = String::default();
+        source_file.read_to_string(&mut updated)?;
+        Ok(updated)
+    }
 }
