@@ -1,8 +1,9 @@
-windows::runtime::include_bindings!();
+#![allow(non_snake_case)]
+pub mod Microsoft;
+
+pub(crate) use ::windows as Windows;
 
 pub mod browser_version;
-
-#[cfg(windows)]
 pub mod callback_interfaces;
 
 #[cfg(test)]
